@@ -45,7 +45,7 @@ class QRydBackend(Backend):
 
     """
 
-    url_base = "https://api.qryddemo.itp3.uni-stuttgart.de/v2_0/jobs"
+    url_base = "https://api.qryddemo.itp3.uni-stuttgart.de/v3_0/jobs"
     """URL to the `web API
     <https://api.qryddemo.itp3.uni-stuttgart.de/docs>`_ endpoint for submitting
     simulation jobs to QRydDemo's cloud emulator."""
@@ -64,8 +64,14 @@ class QRydBackend(Backend):
         self.options.set_validator("shots", (1, 2**18))
         self.options.set_validator("memory", [False])
         self.options.set_validator("seed_simulator", int)
-        self.options.set_validator("develop", bool)
+        self.options.set_validator("seed_compiler", int)
+        self.options.set_validator("dev", bool)
         self.options.set_validator("fusion_max_qubits", (2, 6))
+        self.options.set_validator("use_extended_set", bool)
+        self.options.set_validator("use_reverse_traversal", bool)
+        self.options.set_validator("extended_set_size", (1, 100))
+        self.options.set_validator("extended_set_weight", float)
+        self.options.set_validator("reverse_traversal_iterations", (1, 30))
 
     @property
     def target(self) -> Target:
@@ -101,33 +107,61 @@ class QRydBackend(Backend):
             shots=1024,
             memory=False,
             seed_simulator=None,
-            develop=False,
+            seed_compiler=None,
+            dev=False,
             fusion_max_qubits=4,
+            use_extended_set=True,
+            use_reverse_traversal=True,
+            extended_set_size=4,
+            extended_set_weight=0.5,
+            reverse_traversal_iterations=3,
         )
 
     def set_option(
-        self, key: str, value: Union[Optional[int], Optional[bool], Optional[str]]
+        self,
+        key: str,
+        value: Union[Optional[int], Optional[bool], Optional[str], Optional[float]],
     ) -> None:
         r"""Set an option.
 
         Args:
             key: The key of the option. Currently, the following options are supported:
 
-                * :code:`shots` (:external:py:class:`int`):
+                * :code:`shots` (:external:py:class:`int`, default: 1024):
                   Number of measurements, must be :math:`\geq 1` and
                   :math:`\leq 2^{18}`.
-                * :code:`seed_simulator` (:external:py:class:`int`):
+                * :code:`seed_simulator` (:external:py:class:`int`, default: None):
                   A seed for the random number generator of the emulator.
-                * :code:`fusion_max_qubits` (:external:py:class:`int`):
+                * :code:`fusion_max_qubits` (:external:py:class:`int`, default: 4):
                   The maximum number of qubits that can be fused in a single unitary.
-                * :code:`develop` (:external:py:class:`bool`):
-                  Whether to use the develop version of the emulator.
+                * :code:`dev` (:external:py:class:`bool`, default: False):
+                  Whether to use the development version of the emulator.
+
+                If the backend supports compilation in the cloud, the following
+                additional options are supported:
+
+                * :code:`seed_compiler` (:external:py:class:`int`, default: None):
+                  A seed for the random number generator of the compiler.
+                * :code:`use_extended_set` (:external:py:class:`bool`, default: True):
+                  Whether the `SABRE algorithm`_ uses the extended set of gates.
+                * :code:`use_reverse_traversal` (:external:py:class:`bool`, default:
+                  True): Whether SABRE uses the reverse traversal
+                  technique to update the initial mapping.
+                * :code:`extended_set_size` (:external:py:class:`int`, default: 4):
+                  Size of the extended set of gates used by SABRE.
+                * :code:`extended_set_weight` (:external:py:class:`float`, default:
+                  0.5):
+                  Weight of the extended set of gates used by SABRE.
+                * :code:`reverse_traversal_iterations` (:external:py:class:`int`,
+                  default: 3): The number of times SABRE is run back and forth.
 
             value: The value of the option.
 
         Raises:
             NotImplementedError: If `key` does not describe a valid option.
 
+        .. _SABRE algorithm: https://arxiv.org/abs/1809.02573
+
         .. # noqa: DAR101 value
 
         """
@@ -163,16 +197,32 @@ class QRydBackend(Backend):
                 )
 
         options = self.options
-        if "develop" in kwargs:
-            options.update_options(develop=kwargs["develop"])
+        if "dev" in kwargs:
+            options.update_options(dev=kwargs["dev"])
         if "seed_simulator" in kwargs:
             options.update_options(seed_simulator=kwargs["seed_simulator"])
+        if "seed_compiler" in kwargs:
+            options.update_options(seed_compiler=kwargs["seed_compiler"])
         if "memory" in kwargs:
             options.update_options(memory=kwargs["memory"])
         if "shots" in kwargs:
             options.update_options(shots=kwargs["shots"])
         if "fusion_max_qubits" in kwargs:
             options.update_options(fusion_max_qubits=kwargs["fusion_max_qubits"])
+        if "use_extended_set" in kwargs:
+            options.update_options(use_extended_set=kwargs["use_extended_set"])
+        if "use_reverse_traversal" in kwargs:
+            options.update_options(
+                use_reverse_traversal=kwargs["use_reverse_traversal"]
+            )
+        if "extended_set_size" in kwargs:
+            options.update_options(extended_set_size=kwargs["extended_set_size"])
+        if "extended_set_weight" in kwargs:
+            options.update_options(extended_set_weight=kwargs["extended_set_weight"])
+        if "reverse_traversal_iterations" in kwargs:
+            options.update_options(
+                reverse_traversal_iterations=kwargs["reverse_traversal_iterations"]
+            )
 
         job_dict = self._convert_to_wire_format(circuit, options)
         job_handle = self._submit_to_backend(job_dict, self._provider.session)
@@ -241,17 +291,18 @@ class QRydBackend(Backend):
                     raise AssertionError(
                         "Number of qubits and classical bits must be same."
                     )
-                circuit_dict["ClassicalRegister"]["measurement"]["circuits"][0][
-                    "operations"
-                ] += [
-                    {
-                        "PragmaRepeatedMeasurement": {
-                            "readout": "ro",
-                            "number_measurements": options.shots,
-                            "qubit_mapping": dict(zip(qubits, clbits)),
+                for qubit, clbit in zip(qubits, clbits):
+                    circuit_dict["ClassicalRegister"]["measurement"]["circuits"][0][
+                        "operations"
+                    ] += [
+                        {
+                            "MeasureQubit": {
+                                "readout": "ro",
+                                "qubit": qubit,
+                                "readout_index": clbit,
+                            }
                         }
-                    }
-                ]
+                    ]
             elif inst.name == "p":
                 if len(qubits) != 1 or len(params) != 1:
                     raise AssertionError("Wrong number of arguments.")
@@ -428,12 +479,29 @@ class QRydBackend(Backend):
             else:
                 raise RuntimeError("Operation '%s' not supported." % inst.name)
 
+        circuit_dict["ClassicalRegister"]["measurement"]["circuits"][0][
+            "operations"
+        ] += [
+            {
+                "PragmaSetNumberOfMeasurements": {
+                    "readout": "ro",
+                    "number_measurements": options.shots,
+                }
+            }
+        ]
+
         job_dict = {
             "backend": self.name,
-            "develop": options.develop,
+            "dev": options.dev,
             "fusion_max_qubits": options.fusion_max_qubits,
-            "seed": options.seed_simulator,
+            "seed_simulator": options.seed_simulator,
+            "seed_compiler": options.seed_compiler,
             "pcz_theta": float(PCZGate().get_theta()),
+            "use_extended_set": options.use_extended_set,
+            "use_reverse_traversal": options.use_reverse_traversal,
+            "extended_set_size": options.extended_set_size,
+            "extended_set_weight": options.extended_set_weight,
+            "reverse_traversal_iterations": options.reverse_traversal_iterations,
             "program": circuit_dict,
         }
         return job_dict
